#
# ripecheck.tcl  Version: 0.5  Author: Stefan Wold <ratler@gmail.com>
###
# Info: 
# This script check unresolved ip addresses against a RIPE database
# and ban the user if the country match your configured topdomains.
###
# Usage:
# Simply load the script and change the topdomains you
# wish to ban. You can test ip addresses from dcc console by
# enabling debug output (.console +d) and running .testripecheck ip
#
# chanset <channel> <+/->ripecheck
# This will either enable (+) or disable (-) the script for the 
# specified channel
#
# chanset <channel> ripecheck.bantime <number>
# For how long should the ban be active in minutes
#
###
# Tested:
# eggdrop v1.6.18 GNU/Linux with tcl 8.4 and http 2.5
# eggdrop v1.6.17 GNU/Linux with tcl 8.4 and http 2.5
###
# TODO:
# - Per channel settings
# - Change configuration through dcc console
# - Perhaps add other databases like APNIC etc etc?  
###
# ChangeLog:
# 0.5: Added better error handling during the http query.
#      Added channel flags so it's possible to enable/disable
#      ripecheck per channel.
# 0.4: Changed to using http package instead of sockets.
#      Added configuration option for ban time 
# 0.2: First release (not public)
###
# LICENSE:
# Copyright (C) 2006  Stefan Wold <ratler@gmail.com>
#
# This code comes with ABSOLUTELY NO WARRANTY
#                                                                           
# This program is free software; you can redistribute it and/or modify      
# it under the terms of the GNU General Public License as published by      
# the Free Software Foundation; either version 2 of the License, or         
# (at your option) any later version.                                       
#                                                                           
# This program is distributed in the hope that it will be useful,           
# but WITHOUT ANY WARRANTY; without even the implied warranty of            
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
# GNU General Public License for more details.                              
#                                                                           
# You should have received a copy of the GNU General Public License         
# along with this program; if not, write to the Free Software               
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# RIPE Country Checker

# --- Settings ---

# Space separated list of topdomains you want to ban, see example below
#set topdomains { "ro" "ma" "tr" }
set topdomains { }

# ---- Only edit stuff below this line if you know what you are doing ----
set ver "0.5"

# Channel flags
setudef flag ripecheck
setudef int ripecheck.bantime

# Packages
package require http 2.5

# Bindings
bind join - *!*@* ripecheck_onjoin
bind dcc -|- testripecheck testripecheck

# Functions
proc ripecheck_onjoin { nick host handle channel } {
    # Only run if channel is defined
    if {![channel get $channel ripecheck]} { return 0 }
    
    # Exclude ops, voice, friends
    if {[matchattr $handle fov|fov $channel]} { 
	putloglev d * "ripecheck: $nick is on exempt list"
	return 0 
    }

    # Only run RIPE check on numeric IP
    regexp ".+@(.+)" $host matches iphost
    if [regexp {[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$} $iphost] {
	putloglev d * "ripecheck: Found numeric IP $iphost ... scanning"
	ripecheck $nick $host $iphost $channel
    }
}

proc ripecheck { nick host iphost channel } {
    global topdomains
    set ripe [get_html $iphost]
    set bantime [channel get $channel ripecheck.bantime]
    foreach country $topdomains {
	if {![string compare $ripe $country]} {
	    putlog "ripecheck: Matched country '$ripe' banning $nick!$host for $bantime minute(s)"
	    newchanban $channel "*!*@$iphost" ripecheck "RIPE Country Check: Matched .$ripe" $bantime
	}
    }
}

proc testripecheck { nick idx args } {
    global topdomains
    set ip [lindex [split $args] 0]
    if {![regexp {[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$} $ip]} { return 0 }

    set ripe [get_html $ip]
    foreach country $topdomains {
	if {![string compare $ripe $country]} {
	    putloglev d * "ripecheck: Matched '$ripe' for $ip"
	}
    }
}

proc get_html { iphost } {
    set query "http://www.ripe.net/fcgi-bin/whois?form_type=simple&full_query_string=&searchtext=${iphost}&submit.x=0&submit.y=0"
    set useragent "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.5) Gecko/20040120 Firebird/0.7"
    set http [::http::config -useragent $useragent]
    catch {set http [::http::geturl $query -timeout [expr 1000 * 5]]} error
    
    if {[::http::status $http] == "timeout"} {
	putlog "ripecheck: Error - Connection timed out!"
	::http::cleanup $http
	return 0
    }
    if {[string match -nocase "*couldn't open socket*" $error]} {
	putlog "ripecheck: Error - Connection to ripe failed!"
	::http::cleanup $http
	return 0
    }

    set output [::http::data $http]
    ::http::cleanup $http
 
    if {[regexp -line -nocase {country:\s*([a-z]{2,4})} $output -> line]} {
	set line [string tolower $line]
	putloglev d * "ripecheck: DEBUG - RIPE answer: $line"
	return $line
    }
    return ""
}

putlog "ripecheck v$ver by Ratler loaded"
